import pygame as pg           # main Pygame module as pg
import sys                    # system module for exiting the game
import time                   # time module for delaying the game

# define variables
# colors
WHITE = (255,255,255)
BLACK = (0,0,0)
RED = (255,0,0)

# game settings
WIDTH, HEIGHT = 400, 500                # game screen size
BOARD_HEIGHT = HEIGHT - 100             # board height
RUN = True
turn = 'X'                              # player's turn
FPS = 30                                # game refresh rate
board = [[None]*3 for i in range(3)]    # board array

# initial game setup
pg.init()
screen = pg.display.set_mode((WIDTH, HEIGHT))
pg.display.set_caption('Tic Tac Toe (TutorialsTonight)')

# game images
x_image = pg.image.load('x.png')
o_image = pg.image.load('o.png')
intro_image = pg.image.load('tic-tac-toe-intro.png')

# tranform image to desired size
x_image = pg.transform.scale(x_image, (80, 80))
o_image = pg.transform.scale(o_image, (80, 80))
intro_image = pg.transform.scale(intro_image, (WIDTH, HEIGHT))

# game intro
def game_intro():
    screen.blit(intro_image, (0,0))
    pg.display.update()
    time.sleep(1)
    screen.fill(WHITE)

# draw board lines
def draw_line():
    pg.draw.line(screen, BLACK, (WIDTH/3, 10),(WIDTH/3, BOARD_HEIGHT - 10), 3)
    pg.draw.line(screen, BLACK, (WIDTH*2/3, 10),(WIDTH*2/3, BOARD_HEIGHT - 10), 3)
    pg.draw.line(screen, BLACK, (10, BOARD_HEIGHT/3),(WIDTH - 10, BOARD_HEIGHT/3), 3)
    pg.draw.line(screen, BLACK, (10, BOARD_HEIGHT*2/3),(WIDTH - 10, BOARD_HEIGHT*2/3), 3)

# Show status of game (Whose Turn or Who Won)
def show_status(message):
    font = pg.font.Font(None, 30)
    # draw a rectangle at the bottom of the screen
    pg.draw.rect(screen, BLACK, (0, BOARD_HEIGHT, WIDTH, 100))

    if message == 'X':
        status_text = font.render("X's turn", True, WHITE)
    elif message == 'O':
        status_text = font.render("O's turn", True, WHITE)
    else:
        status_text = font.render(message, True, WHITE)

    screen.blit(status_text, ((WIDTH - status_text.get_width())/2, BOARD_HEIGHT + 50))
    pg.display.update()

# function to handle player's move
def player_turn(x, y):
    global turn

    # return if x,y is out of bounds
    if x < 10 or x > WIDTH - 20 or y < 10 or y > BOARD_HEIGHT - 20:
        return

    # get index of box clicked
    x_index = int(x // (WIDTH/3))
    y_index = int(y // (BOARD_HEIGHT/3))

    # check if box is already filled
    if board[x_index][y_index] is not None:
        return

    # get position on board to draw x or o
    PADDING = 30
    if board[x_index][y_index] == None:
        if x_index == 0:
            if y_index == 0:
                x_pos = PADDING
                y_pos = PADDING
            elif y_index == 1:
                x_pos = PADDING
                y_pos = BOARD_HEIGHT/3 + PADDING
            else:
                x_pos = PADDING
                y_pos = BOARD_HEIGHT*2/3 + PADDING
        elif x_index == 1:
            if y_index == 0:
                x_pos = WIDTH/3 + PADDING
                y_pos = PADDING
            elif y_index == 1:
                x_pos = WIDTH/3 + PADDING
                y_pos = BOARD_HEIGHT/3 + PADDING
            else:
                x_pos = WIDTH/3 + PADDING
                y_pos = BOARD_HEIGHT*2/3 + PADDING
        elif x_index == 2:
            if y_index == 0:
                x_pos = WIDTH*2/3 + PADDING
                y_pos = PADDING
            elif y_index == 1:
                x_pos = WIDTH*2/3 + PADDING
                y_pos = BOARD_HEIGHT/3 + PADDING
            else:
                x_pos = WIDTH*2/3 + PADDING
                y_pos = BOARD_HEIGHT*2/3 + PADDING

    if turn == 'X':
        screen.blit(x_image, (x_pos, y_pos))
        board[x_index][y_index] = 'X'
        turn = 'O'
        show_status(turn)
        check_winner('X')
    else:
        screen.blit(o_image, (x_pos, y_pos))
        board[x_index][y_index] = 'O'
        turn = 'X'
        show_status(turn)
        check_winner('O')


# close game
def close_game():
    time.sleep(1.5)
    pg.quit()
    sys.exit()

# check if there is a winner
def check_winner(turn):
    # check row and column
    for i in range(3):
        # checking row
        if board[0][i] == turn and board[1][i] == turn and board[2][i] == turn:
            pg.draw.line(screen, RED, (10, (2*i+1)*BOARD_HEIGHT/6),(WIDTH - 10, (2*i+1)*BOARD_HEIGHT/6), 3)
            show_status(turn + ' Wins!')
            close_game()
        # checking column
        if board[i][0] == turn and board[i][1] == turn and board[i][2] == turn:
            pg.draw.line(screen, RED, ((2*i+1)*WIDTH/6, 10),((2*i+1)*WIDTH/6, BOARD_HEIGHT - 10), 3)
            show_status(turn + ' Wins!')
            close_game()
    # check diagonal
    if board[0][0] == turn and board[1][1] == turn and board[2][2] == turn:
        pg.draw.line(screen, RED, (10, 10), (WIDTH - 10, BOARD_HEIGHT - 10), 5)
        show_status(turn + ' Wins!')
        close_game()
    if board[0][2] == turn and board[1][1] == turn and board[2][0] == turn:
        pg.draw.line(screen, RED, (10, BOARD_HEIGHT - 10), (WIDTH - 10, 10), 5)
        show_status(turn + ' Wins!')
        close_game()
    # check if board is full print tie
    boxes = 0
    for i in range(3):
        for j in range(3):
            if board[i][j] is not None:
                boxes += 1
    if boxes == 9:
        show_status("It's a Tie!")
        close_game()

    return False

# start the game
def start_game():
    game_intro()
    draw_line()
    show_status(turn)
    pg.display.update()

start_game()

# main game loop
while RUN:
    for event in pg.event.get():
        if event.type == pg.QUIT:
            RUN = False
            sys.exit()
        if event.type == pg.MOUSEBUTTONDOWN:
            x, y = pg.mouse.get_pos()
            player_turn(x, y)

    pg.display.update()
    pg.time.Clock().tick(FPS)
